<?php
/** 
*
* @package phpBB3
* @version $Id: smartfeed_url.php, v 2.0.5 Production 2008/05/18 mark@phpbbservices.com Exp $
* @copyright (c) Mark D. Hamill (mhamill@computer.org, http://phpbbservices.com) 
* @license http://opensource.org/licenses/gpl-license.php GNU Public License 
*
*/

/**
* @ignore
*/

// Written by Mark D. Hamill, mhamill@computer.org, http://phpbbservices.com
// This software is designed to work with phpBB Version 3.0.3

// This is the user interface for the Smartfeed software. Users can use it to create a URL they can use 
// with newsfeed readers like FeedReader or web based personal news aggregators like Google Reader.
// Unlike most RSS newsfeeds this one can dip into member-only and user group permission based
// forums which should not be accessible to the world and return posts in these topics. It handles
// RSS and Atom feeds.

define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);
require_once($phpbb_root_path . 'includes/constants_smartfeed.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup('mods/smartfeed');

// Get the session ID. Since it may not be on the URL field, get it from the database. Getting it enables
// helps a user to create a new Smartfeed key, providing they are logged in.
$sql = 'SELECT session_id
		FROM ' . SESSIONS_TABLE . '
		WHERE session_user_id = ' . $user->data['user_id'] . " AND session_ip = '" . getenv('REMOTE_ADDR') . "'
		ORDER BY session_start DESC";
$result = $db->sql_query($sql);
$row = $db->sql_fetchrow($result);
$session_id	= $row['session_id'];

// Smartfeed cannot be used with Apache authentication unless the .htaccess file is modified to allow smartfeed.php to bypass
// Apache authentication. If you have made these changes then set the constant SMARTFEED_APACHE_HTACCESS_ENABLED to true.
if (($config['auth_method'] == 'apache') && ($config['sf_apache_htaccess_enabled'] != 1))
{
	$msg_text = ($user->data['user_type'] == USER_FOUNDER) ? $user->lang['SMARTFEED_APACHE_AUTHENTICATION_WARNING_ADMIN'] : $user->lang['SMARTFEED_ERROR']['SMARTFEED_APACHE_AUTHENTICATION_WARNING_REG'];
	trigger_error($msg_text, E_USER_NOTICE);
}

// Create a full site URL. 
$board_url = generate_board_url() . '/';
$registered_user_mode = false;

if (($config['auth_method'] != 'ldap') && ($user->data['user_id'] != ANONYMOUS) && (($user->data['user_type'] == USER_NORMAL) || ($user->data['user_type'] == USER_FOUNDER)))
{
	// If the user is registered then great, they can authenticate and see private forums
	$smartfeed_user_id = $user->data['user_id'];
	$user_password = $user->data['user_password'];
	if (($user->data['user_smartfeed_key']))
	{
		$user_smartfeed_key = $user->data['user_smartfeed_key'];
		$encrypted_password = encrypt($user_password, $user_smartfeed_key);
		$encrypted_password_with_ip = encrypt($user_password . '~' . getenv('REMOTE_ADDR'), $user_smartfeed_key);
	}
	else
	{
		// Generate a smartfeed encryption key. This is a one time action. It is used to authenticate the user when they call smartfeed.php.
		$user_smartfeed_key = gen_rand_string(32);
		$encrypted_password = encrypt($user_password, $user_smartfeed_key);
		$encrypted_password_with_ip = encrypt($user_password . '~' . getenv('REMOTE_ADDR'), $user_smartfeed_key);
		
		// Store the key
		$sql = 'UPDATE ' . USERS_TABLE . "
			   SET user_smartfeed_key = '" . $db->sql_escape($user_smartfeed_key) . "'
			   WHERE user_id = " . intval($user->data['user_id']);
		$result = $db->sql_query($sql);
	}
	$registered_user_mode = true;
	$template->assign_vars(array('S_IS_GUEST' => 'F', 'L_DAY_DEFAULT' => ''));
}
else
{
	// Public (anonymous) users do not need to authenticate so no encrypted passwords are needed
	$smartfeed_user_id = ANONYMOUS;
	$encrypted_password = 'NONE';
	$encrypted_password_with_ip = 'NONE';
	$template->assign_vars(array('L_DAY_DEFAULT' => 'selected="selected"'));
	// Make sure to show a message that the user cannot select from read protected forums
	if ($config['auth_method'] != 'ldap')
	{
		// Note: If LDAP authentication is enabled, then no users can access private forums as a feed, so this message
		// is not needed.
		$template->assign_vars(array('S_IS_GUEST' => 'T'));
	}
}

// If IP Authentication is required, hide HTML that solicits whether the user wants to use IP authentication or not ... they have no choice!
if (($config['sf_require_ip_authentication'] == 1) && $registered_user_mode)
{
	$template->assign_vars(array('S_REQUIRED_IP_AUTHENTICATION' => 'T'));	
}
else if ($registered_user_mode)
{
	$template->assign_vars(array('S_REQUIRED_IP_AUTHENTICATION' => 'F',
		'L_IP_AUTHENTICATION' => $user->lang['SMARTFEED_IP_AUTH']));	
}

$forum_read_ary = array();
$allowed_forums = array();

// Get a list of forums for which this user has read permissions
if ($config['auth_method'] == 'ldap')

{

	// For LDAP authentication, since there is no PHP method to retrieve the user's password from a LDAP directory (or even a hash), we cannot
	// create an authentication string. Thus we are limited to getting newsfeeds for public forums only.
	
	$template->assign_vars(array('S_LDAP' => 'T'));

	// First, get the user_permissions column for Anonymous
	$sql = 'SELECT user_permissions
			FROM ' . USERS_TABLE . '
			WHERE user_id = ' . ANONYMOUS;
	$result = $db->sql_query($sql);
	$row = $db->sql_fetchrow($result);
	$user_permissions = $row['user_permissions'];
	$db->sql_freeresult($result);

	// Begin code stolen and tweaked from auth.php
	
	$sql = 'SELECT auth_option, is_global, is_local
			FROM ' . ACL_OPTIONS_TABLE . '
			ORDER BY auth_option_id';
	$db->sql_query($sql);

	$local = 0;
	$acl_options = array();
	$acl = array();
	while ($row = $db->sql_fetchrow($result))
	{
		if ($row['is_local'])
		{
			$acl_options['local'][$row['auth_option']] = $local++;
		}
	}
	$db->sql_freeresult($result);

	$user_permissions = explode("\n", $user_permissions);
	
	if (sizeof($user_permissions) > 0) // This should avoid a PHP Notice
	{
		foreach ($user_permissions as $f => $seq)
		{
			if ($seq)
			{
				$i = 0;
		
				if (!isset($acl[$f]))
				{
					$acl[$f] = '';
				}
		
				while ($subseq = substr($seq, $i, 6))
				{
					// We put the original bitstring into the acl array
					$acl[$f] .= str_pad(base_convert($subseq, 36, 2), 31, 0, STR_PAD_LEFT);
					$i += 6;
				}
			}
		}
	}
	
	// End code stolen from auth.php

	if (sizeof($acl_options['local']) > 0) // This should avoid a PHP Notice
	{
		$f_read_local_index = -1;
		foreach ($acl_options['local'] as $key => $value)
		{
			if ($key == 'f_read')
			{
				$f_read_local_index = $value;
				break;
			}
		}
	}

	// Create the allowed forums array. These are the forums for which this user has read privileges. He can select
	// ONLY from these forums. If they request they can choose from a smaller subset of these forums.
	
	if (sizeof($acl) > 0) // This should avoid a PHP Notice
	{
		foreach ($acl as $key => $value)
		{
			if (substr($value, $f_read_local_index, 1) == '1')
			{
				$forum_read_ary[$key]['f_read'] = 1;
			}
		}
	}
	
}

else

{
	// If the user can be authenticated then we can get a list of allowed forums very easily by calling acl_getf()
	$forum_read_ary = $auth->acl_getf('f_read');
}

// Get a list of parent_ids for each forum and put them in an array.
$parent_array = array();
$sql = 'SELECT forum_id, parent_id 
	FROM ' . FORUMS_TABLE . '
	ORDER BY 1';
$result = $db->sql_query($sql);
while ($row = $db->sql_fetchrow($result))
{
	$parent_array[$row['forum_id']] = $row['parent_id'];
}

if (sizeof($forum_read_ary ) > 0) // This should avoid a PHP Notice
{
	foreach ($forum_read_ary as $forum_id => $allowed)
	{
		if ($allowed['f_read'])
		{
			// Since this user has read access to this forum, add it to the $allowed_forums array
			$allowed_forums[] = (int) $forum_id;
			
			// Also add to $allowed_forums the parents, if any, of this forum. Actually we have to find the parent's parents, etc., going up as far as necessary because 
			// $auth->act_getf does not return the parents for which the user has access, yet parents must be shown are on the interface
			$there_are_parents = true;
			$this_forum_id = (int) $forum_id;
			
			while ($there_are_parents)
			{
				if ($parent_array[$this_forum_id] == 0)
				{
					$there_are_parents = false;
				}
				else
				{
					// Do not add this parent to the list of allowed forums if it is already in the array
					if (!in_array((int) $parent_array[$this_forum_id], $allowed_forums))
					{
						$allowed_forums[] = (int) $parent_array[$this_forum_id];
					} 
					$this_forum_id = (int) $parent_array[$this_forum_id];	// Keep looping...
				}
			}
		}
	}
}

// Get a list of forums as they appear on the main index for this user. For presentation purposes indent them so they show the natural phpBB3 hierarchy.
// Indenting is cleverly handled by nesting <div> tags inside of other <div> tags, and the template defines the relative offset (20 pixels).

if (sizeof($allowed_forums) > 0)

{

	$sql = 'SELECT forum_name, forum_id, parent_id, forum_type
			FROM ' . FORUMS_TABLE . ' 
			WHERE ' . $db->sql_in_set('forum_id', $allowed_forums) . ' AND forum_type != ' . FORUM_LINK . '
			ORDER BY left_id ASC';
	$result = $db->sql_query($sql);
	
	$template->assign_block_vars('show_forums', array());
	
	$current_level = 0;			// How deeply nested are we at the moment
	$parent_stack = array();	// Holds a stack showing the current parent_id of the forum
	$parent_stack[] = 0;		// 0, the first value in the stack, represents the <div_0> element, a container holding all the categories and forums in the template
	
	while ($row = $db->sql_fetchrow($result))
	{
	
		if ((int) $row['parent_id'] != (int) end($parent_stack) || (end($parent_stack) == 0))
		{
			if (in_array($row['parent_id'],$parent_stack))
			{
				// If parent is in the stack, then pop the stack until the parent is found, otherwise push stack adding the current parent. This creates a </div>
				while ((int) $row['parent_id'] != (int) end($parent_stack))
				{
					array_pop($parent_stack);
					$current_level--;
					// Need to close a category level here
					$template->assign_block_vars('forums', array( 
						'DIV_OPEN_CLOSE' => 'C',
						'PRINT' => '0'));
				}
			}
			else
			{
				// If the parent is not in the stack, then push the parent_id on the stack. This is also a trigger to indent the block. This creates a <div>
				array_push($parent_stack, (int) $row['parent_id']);
				$current_level++;
				// Need to add a category level here
				$template->assign_block_vars('forums', array( 
					'DIV_OPEN_CLOSE' => 'O',
					'CAT_ID' => 'div_' . $row['parent_id'],
					'PRINT' => '0'));
			}
		}
		
		// This code prints the forum or category, which will exist inside the previously created <div> block
		$template->assign_block_vars('forums', array( 
			'DIV_OPEN_CLOSE' => 'X',
			'FORUM_NAME' => 'elt_' . (int) $row['forum_id'] . '_' . (int) $row['parent_id'],
			'FORUM_LABEL' => $row['forum_name'],
			'PRINT' => '1',
			'IS_FORUM' => ($row['forum_type'] == FORUM_CAT) ? '0' : '1'));
		
	}

	$db->sql_freeresult($result);
	
	// Now out of the loop, it is important to remember to close any open <div> tags. Typically there is at least one.
	while ((int) $row['parent_id'] != (int) end($parent_stack))
	{
		array_pop($parent_stack);
		$current_level--;
		// Need to close the <div> tag
		$template->assign_block_vars('forums', array( 
			'DIV_OPEN_CLOSE' => 'C',
			'PRINT' => '0'));
	}
	
	// Now to output most of the lovely text to the screen
	$template->assign_vars(array(
		'ALL_BY_DEFAULT'			=> ($config['sf_all_by_default'] == '1') ? 'checked="checked"' : '',
		'L_1_HRS_VALUE'				=> SMARTFEED_LAST_1_HOURS_VALUE,
		'L_12_HRS_VALUE'			=> SMARTFEED_LAST_12_HOURS_VALUE,
		'L_15_MIN_VALUE'			=> SMARTFEED_LAST_15_MINUTES_VALUE,
		'L_3_HRS_VALUE'				=> SMARTFEED_LAST_3_HOURS_VALUE,
		'L_30_MIN_VALUE'			=> SMARTFEED_LAST_30_MINUTES_VALUE,
		'L_6_HRS_VALUE'				=> SMARTFEED_LAST_6_HOURS_VALUE,
		'L_BASIC_VALUE'				=> SMARTFEED_BASIC,
		'L_BOARD'					=> SMARTFEED_BOARD,
		'L_COMPACT_VALUE'			=> SMARTFEED_COMPACT,
		'L_COUNT_LIMIT_EXPLAIN_MAX' => ($config['sf_max_items'] == 0) ? $user->lang['SMARTFEED_COUNT_LIMIT_EXPLAIN_BLANK'] : sprintf($user->lang['SMARTFEED_COUNT_LIMIT_EXPLAIN'], $config['sf_max_items']),
		'L_COUNT_LIMIT_EXPLAIN_MAX'	=> sprintf($user->lang['SMARTFEED_COUNT_LIMIT_EXPLAIN'], $config['sf_max_items']),
		'L_DAY_VALUE'				=> SMARTFEED_LAST_DAY_VALUE,
		'L_HTML_VALUE'				=> SMARTFEED_HTML,
		'L_HTMLSAFE_VALUE'			=> SMARTFEED_HTMLSAFE,
		'L_LAST_FETCH_VALUE'		=> SMARTFEED_SINCE_LAST_VISIT,
		'L_MONTH_VALUE'				=> SMARTFEED_LAST_MONTH_VALUE,
		'L_NO_LIMIT_VALUE' 			=> SMARTFEED_NO_LIMIT_VALUE,
		'L_POSTDATE_ASCENDING'		=> SMARTFEED_POSTDATE,
		'L_POSTDATE_DESCENDING'		=> SMARTFEED_POSTDATE_DESC,
		'L_QUARTER_VALUE'			=> SMARTFEED_LAST_QUARTER_VALUE,
		'L_SMARTFEED_LIMIT_SET_EXPLAIN'	=> ($config['sf_default_fetch_time_limit'] == '0') ? '' : sprintf($user->lang['SMARTFEED_LIMIT_SET_EXPLAIN'], round(($config['sf_default_fetch_time_limit']/24), 0)),
		'L_STANDARD'				=> SMARTFEED_STANDARD,
		'L_STANDARD_DESC'			=> SMARTFEED_STANDARD_DESC,
		'L_TWO_WEEKS_VALUE'			=> SMARTFEED_LAST_TWO_WEEKS_VALUE,
		'L_WEEK_VALUE'				=> SMARTFEED_LAST_WEEK_VALUE,
		'NO_FORUMS'					=> 'F', 
		'S_ATOM_10_VALUE'			=> $config['sf_atom_10_value'],
		'S_PHPEXT'					=> $phpEx,
		'S_PWD'						=> $encrypted_password, 
		'S_PWD_WITH_IP'				=> $encrypted_password_with_ip, 
		'S_RSS_10_VALUE'			=> $config['sf_rss_10_value'],
		'S_RSS_20_VALUE'			=> $config['sf_rss_20_value'],
		'S_SESSION_ID'				=> $session_id,
		'S_SMARTFEED_MAX_ITEMS'		=> (intval($config['sf_max_items']) == 0) ? '' : $config['sf_max_items'],
		'S_SMARTFEED_PAGE_URL'		=> $config['sf_smartfeed_page_url'],
		'S_SMARTFEED_TITLE'			=> $config['sf_smartfeed_title'],
		'S_SMARTFEED_VERSION'		=> $config['sf_version'],
		'S_USER_ID'					=> $smartfeed_user_id,
		'UA_SITE_URL'				=> $board_url,
	
		)
	);

}
	
else
	
{

	// No forums to show!
	$template->assign_vars(array(
		'NO_FORUMS' => 'T', 
		'L_NO_FORUMS_MESSAGE' =>  $user->lang['SMARTFEED_NO_FORUMS_AVAILABLE']));
	
}

// If the user is an administrator, show latest version information at the bottom of the page
if ($user->data['user_type'] == USER_FOUNDER)
{

	// Check for new version
	include($phpbb_root_path . 'includes/functions_admin.' . $phpEx);
	
	$installed_version = explode('.', $config['sf_version']);
	$major_revision = (int) $installed_version[0]; // Always a number
	$minor_revision = (int) $installed_version[1]; // Always a number
	$micro_revision = $installed_version[2]; // Could be a combination of numbers and letters, ex: RC2
	$letter_revision = (isset($installed_version[3])) ? $installed_version[3] : ''; // If present, a lowercase letter
	
	$version_info = get_remote_file($config['sf_smartfeed_host'], '/smartfeed/updatecheck', 'version_phpBB3.txt', $errstr, $errno);

	if ($version_info === false)
	{
		if ($errstr)
		{
			$version_info = '<span style="color:red">' . sprintf($user->lang['SMARTFEED_CONNECT_SOCKET_ERROR'], $errstr) . '</span>';
		}
		else
		{
			$version_info = '<span>' . $user->lang['SMARTFEED_SOCKET_FUNCTIONS_DISABLED'] . '</span>';
		}
	}
	else
	{
		$version_info = explode("\n", $version_info);
		$latest_head_revision = (int) $version_info[0]; // Always a number
		$latest_minor_revision = (int) $version_info[1]; // Always a number
		$latest_micro_revision = $version_info[2]; // Could be a combination of numbers and letters, ex: RC2
		$latest_letter_revision = (isset($version_info[3])) ? $version_info[3] : ''; // If present, a lowercase letter
	
		if (($latest_head_revision == $major_revision) && ($latest_minor_revision == $minor_revision) && ($latest_micro_revision == $micro_revision) && ($latest_letter_revision == $letter_revision))
		{
			$version_info = '<span style="color:green">' . $user->lang['SMARTFEED_VERSION_UP_TO_DATE'] . '</span>';
		}
		else
		{
			$latest_version = (int) $version_info[0] . '.' . (int) $version_info[1];
			if (strlen($version_info[2]) > 0)
			{
				$latest_version .=  '.' . $version_info[2];
			}
			if (isset($version_info[3]))
			{
				if (strlen($version_info[3]) > 0)
				{
				   $latest_version .=  '.' . $version_info[3];
				}
			}
			$version_info = '<span style="color:red">' . sprintf($user->lang['SMARTFEED_VERSION_NOT_UP_TO_DATE'],$config['sf_smartfeed_page_url']);
			$version_info .= ' ' . sprintf($user->lang['SMARTFEED_LATEST_VERSION_INFO'], $latest_version) . ' ' . sprintf($user->lang['SMARTFEED_CURRENT_VERSION_INFO'], $config['sf_version']) . '</span>';
		}
	}

	$template->assign_vars(array(
		'S_VERSION_INFO' => $version_info,
		'IS_ADMIN' => 'T'));
}

page_header($config['sf_smartfeed_title']);

$template->set_filenames(array(
	'body' => 'smartfeed_url_body.html')
);

make_jumpbox(append_sid("{$phpbb_root_path}viewforum.$phpEx"));

page_footer();

function base64_encode_urlsafe($input)
{
	// Thanks to phpBB forum user klapray for this logic for creating a "urlsafe" versions of base64_encode and _decode.
	return strtr(base64_encode($input), '+/=', '-_.');
}

function encrypt($data_input, $key)
{   

	// This function encrypts $data_input with the given $key. It uses the PHP Secure Communications Library, found
	// at http://phpseclib.sourceforge.net/. If mcrypt is compiled into PHP it will use that, otherwise it will use the functions 
	// from this library. This allows for maximum portability of Smartfeed. Thanks to Terrafrost at phpbb.com for the suggestion
	// to use this library!
	
	global $phpbb_root_path, $phpEx;
	static $des;
	
	if (!isset($des))
	{
		include($phpbb_root_path . "includes/Crypt/TripleDES.$phpEx");
    	$des = new Crypt_TripleDES();
	}

	$des->setKey($key);

	// Encrypt
	$encrypted_data = base64_encode_urlsafe($des->encrypt($data_input));
	return $encrypted_data;

}   

?>